# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.

# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.

# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

from sos.plugins import Plugin, RedHatPlugin
import re
from glob import glob
from datetime import datetime, timedelta


class Cluster(Plugin, RedHatPlugin):
    """cluster suite and GFS related information
    """

    plugin_name = 'cluster'
    option_list = [
        ("gfslockdump", 'gather output of gfs lockdumps', 'slow', False),
        ("crm_from", 'specify the start time for crm_report', 'fast', False),
        ('lockdump', 'gather dlm lockdumps', 'slow', False)
    ]

    packages = [
        "ricci",
        "corosync",
        "openais",
        "cman",
        "clusterlib",
        "fence-agents",
        "pacemaker"
    ]

    files = ["/etc/cluster/cluster.conf"]

    def setup(self):

        self.add_copy_specs([
            "/etc/cluster.conf",
            "/etc/cluster.xml",
            "/etc/cluster",
            "/etc/sysconfig/cluster",
            "/etc/sysconfig/cman",
            "/etc/fence_virt.conf",
            "/var/lib/ricci",
            "/var/lib/luci/data/luci.db",
            "/var/lib/luci/etc",
            "/var/log/cluster",
            "/var/log/luci",
            "/etc/fence_virt.conf",
            "/sys/fs/gfs2/*/withdraw"
        ])

        if self.get_option('gfslockdump'):
            self.do_gfslockdump()

        if self.get_option('lockdump'):
            self.do_lockdump()

        self.add_cmd_outputs([
            "rg_test test /etc/cluster/cluster.conf",
            "fence_tool ls -n",
            "gfs_control ls -n",
            "dlm_tool log_plock",
            "clustat",
            "group_tool dump",
            "cman_tool services",
            "cman_tool nodes",
            "cman_tool status",
            "ccs_tool lsnode",
            "corosync-quorumtool -l",
            "corosync-quorumtool -s",
            "corosync-cpgtool",
            "corosync-objctl",
            "group_tool ls -g1",
            "gfs_control ls -n",
            "gfs_control dump",
            "fence_tool dump",
            "dlm_tool dump",
            "dlm_tool ls -n",
            "mkqdisk -L"
        ])

        # crm_report needs to be given a --from "YYYY-MM-DD HH:MM:SS" start
        # time in order to collect data.
        crm_from = (datetime.today()
                    - timedelta(hours=72)).strftime("%Y-%m-%d %H:%m:%S")
        if self.get_option('crm_from') is not False:
            if re.match(r'\d{4}-\d{2}-\d{2} \d{2}:\d{2}:\d{2}',
                        str(self.get_option('crm_from'))):
                crm_from = self.get_option('crm_from')
            else:
                self._log_error(
                    "crm_from parameter '%s' is not a valid date: using "
                    "default" % self.get_option('crm_from'))

        crm_dest = self.get_cmd_output_path(name='crm_report')
        self.add_cmd_output('crm_report -S -d --dest %s --from "%s"'
                            % (crm_dest, crm_from))

    def do_lockdump(self):
        dlm_tool = "dlm_tool ls"
        result = self.call_ext_prog(dlm_tool)
        if result['status'] != 0:
            return

        lock_exp = r'^name\s+([^\s]+)$'
        lock_re = re.compile(lock_exp, re.MULTILINE)
        for lockspace in lock_re.findall(result['output']):
            self.add_cmd_output(
                "dlm_tool lockdebug -svw '%s'" % lockspace,
                suggest_filename="dlm_locks_%s" % lockspace
            )

    def do_gfslockdump(self):
        mnt_exp = r'^\S+\s+([^\s]+)\s+gfs\s+.*$'
        for mnt in self.do_regex_find_all(mnt_exp, "/proc/mounts"):
            self.add_cmd_output(
                "gfs_tool lockdump %s" % mnt,
                suggest_filename="gfs_lockdump_" + self.mangle_command(mnt)
            )

    def postproc(self):
        for cluster_conf in glob("/etc/cluster/cluster.conf*"):
            self.do_file_sub(
                cluster_conf,
                r"(\s*\<fencedevice\s*.*\s*passwd\s*=\s*)\S+(\")",
                r"\1%s" % ('"***"')
            )

        self.do_path_regex_sub(
            "/var/lib/luci/etc/.*\.ini",
            r"(.*secret\s*=\s*)\S+",
            r"\1******"
        )

        self.do_cmd_output_sub(
            "corosync-objctl",
            r"(.*fence.*\.passwd=)(.*)",
            r"\1******"
        )
        return

# vim: et ts=4 sw=4



# Copyright (C) 2014 Red Hat, Inc., Sandro Bonazzola <sbonazzo@redhat.com>
# Copyright (C) 2013 Chris J Arges <chris.j.arges@canonical.com>
# Copyright (C) 2012-2013 Red Hat, Inc., Bryn M. Reeves <bmr@redhat.com>
# Copyright (C) 2011 Red Hat, Inc., Jesse Jaggars <jjaggars@redhat.com>

# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.

# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.

# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

import os
import tempfile

from sos.plugins import Plugin, RedHatPlugin, UbuntuPlugin, DebianPlugin
from sos.utilities import find


class PostgreSQL(Plugin):
    """PostgreSQL related information"""

    plugin_name = "postgresql"

    packages = ('postgresql',)

    tmp_dir = None

    option_list = [
        ('pghome', 'PostgreSQL server home directory.', '', '/var/lib/pgsql'),
        ('username', 'username for pg_dump', '', 'postgres'),
        ('password', 'password for pg_dump', '', ''),
        ('dbname', 'database name to dump for pg_dump', '', ''),
        ('dbhost', 'database hostname/IP (do not use unix socket)', '', ''),
        ('dbport', 'database server port number', '', '5432')
    ]

    def pg_dump(self):
        dest_file = os.path.join(self.tmp_dir, "sos_pgdump.tar")
        old_env_pgpassword = os.environ.get("PGPASSWORD")
        os.environ["PGPASSWORD"] = self.get_option("password")
        if self.get_option("dbhost"):
            cmd = "pg_dump -U %s -h %s -p %s -w -f %s -F t %s" % (
                self.get_option("username"),
                self.get_option("dbhost"),
                self.get_option("dbport"),
                dest_file,
                self.get_option("dbname")
            )
        else:
            cmd = "pg_dump -C -U %s -w -f %s -F t %s " % (
                self.get_option("username"),
                dest_file,
                self.get_option("dbname")
            )
        result = self.call_ext_prog(cmd)
        if old_env_pgpassword is not None:
            os.environ["PGPASSWORD"] = str(old_env_pgpassword)
        if (result['status'] == 0):
            self.add_copy_spec(dest_file)
        else:
            self._log_error(
                "Unable to execute pg_dump. Error(%s)" % (result['output'])
            )
            self.add_alert(
                "ERROR: Unable to execute pg_dump. Error(%s)" %
                (result['output'])
            )

    def setup(self):
        if self.get_option("dbname"):
            if self.get_option("password"):
                self.tmp_dir = tempfile.mkdtemp()
                self.pg_dump()
            else:
                self.soslog.warning(
                    "password must be supplied to dump a database."
                )
                self.add_alert(
                    "WARN: password must be supplied to dump a database."
                )
        else:
            self.soslog.warning(
                "dbname must be supplied to dump a database."
            )
            self.add_alert(
                "WARN: dbname must be supplied to dump a database."
            )

    def postproc(self):
        import shutil
        if self.tmp_dir:
            try:
                shutil.rmtree(self.tmp_dir)
            except shutil.Error:
                self.soslog.exception(
                    "Unable to remove %s." % (self.tmp_dir)
                )
                self.add_alert("ERROR: Unable to remove %s." % (self.tmp_dir))


class RedHatPostgreSQL(PostgreSQL, RedHatPlugin):
    """PostgreSQL related information for Red Hat distributions"""

    def setup(self):
        super(RedHatPostgreSQL, self).setup()

        # Copy PostgreSQL log files.
        for filename in find("*.log", self.get_option("pghome")):
            self.add_copy_spec(filename)
        # Copy PostgreSQL config files.
        for filename in find("*.conf", self.get_option("pghome")):
            self.add_copy_spec(filename)

        self.add_copy_spec(
            os.path.join(
                self.get_option("pghome"),
                "data",
                "PG_VERSION"
            )
        )
        self.add_copy_spec(
            os.path.join(
                self.get_option("pghome"),
                "data",
                "postmaster.opts"
            )
        )


class DebianPostgreSQL(PostgreSQL, DebianPlugin, UbuntuPlugin):
    """PostgreSQL related information for Debian/Ubuntu distributions"""

    def setup(self):
        super(DebianPostgreSQL, self).setup()

        self.add_copy_specs([
            "/var/log/postgresql/*.log",
            "/etc/postgresql/*/main/*.conf",
            "/var/lib/postgresql/*/main/PG_VERSION",
            "/var/lib/postgresql/*/main/postmaster.opts"
        ])


# vim: et ts=4 sw=4



# Copyright (C) 2007 Sadique Puthen <sputhenp@redhat.com>

# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.

# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.

# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

from sos.plugins import Plugin, RedHatPlugin, DebianPlugin, UbuntuPlugin


class OpenSSL(Plugin):
    """openssl related information
    """

    plugin_name = "openssl"
    packages = ('openssl',)

    def postproc(self):
        protect_keys = [
            "input_password",
            "output_password",
            "challengePassword"
        ]

        regexp = r"(?m)^(\s*#?\s*(%s).*=)(.*)" % "|".join(protect_keys)

        self.do_file_sub(
            '/etc/ssl/openssl.cnf',
            regexp,
            r"\1 ******"
        )


class RedHatOpenSSL(OpenSSL, RedHatPlugin):
    """openssl related information for Red Hat distributions
    """

    files = ('/etc/pki/tls/openssl.cnf',)

    def setup(self):
        super(RedHatOpenSSL, self).setup()
        self.add_copy_spec("/etc/pki/tls/openssl.cnf")


class DebianOpenSSL(OpenSSL, DebianPlugin, UbuntuPlugin):
    """openssl related information for Debian distributions
    """

    files = ('/etc/ssl/openssl.cnf',)

    def setup(self):
        super(DebianOpenSSL, self).setup()
        self.add_copy_spec("/etc/ssl/openssl.cnf")

# vim: et ts=4 sw=4



# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.

# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.

# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

import os
from sos.plugins import Plugin, RedHatPlugin, DebianPlugin, UbuntuPlugin


class Logs(Plugin):
    """log data """

    plugin_name = "logs"

    def setup(self):
        self.add_copy_specs([
            "/etc/syslog.conf",
            "/etc/rsyslog.conf"
        ])

        self.limit = self.get_option("log_size")
        self.add_copy_spec_limit("/var/log/boot.log", sizelimit=self.limit)
        self.add_copy_spec_limit("/var/log/cloud-init*", sizelimit=self.limit)

        if self.get_option('all_logs'):
            logs = self.do_regex_find_all("^\S+\s+(-?\/.*$)\s+",
                                          "/etc/syslog.conf")
            if self.is_installed("rsyslog") \
                    or os.path.exists("/etc/rsyslog.conf"):
                logs += self.do_regex_find_all("^\S+\s+(-?\/.*$)\s+",
                                               "/etc/rsyslog.conf")
            for i in logs:
                if i.startswith("-"):
                    i = i[1:]
                if os.path.isfile(i):
                    self.add_copy_spec_limit(i, sizelimit=self.limit)


class RedHatLogs(Logs, RedHatPlugin):
    """Basic system information for RedHat based distributions"""

    def setup(self):
        super(RedHatLogs, self).setup()
        self.add_copy_spec_limit("/var/log/secure*", sizelimit=self.limit)
        self.add_copy_spec_limit("/var/log/messages*", sizelimit=self.limit)


class DebianLogs(Logs, DebianPlugin, UbuntuPlugin):
    """Basic system information for Debian and Ubuntu based distributions"""

    def setup(self):
        super(DebianLogs, self).setup()
        self.add_copy_specs([
            "/var/log/syslog",
            "/var/log/udev",
            "/var/log/kern*",
            "/var/log/mail*",
            "/var/log/dist-upgrade",
            "/var/log/installer",
            "/var/log/unattended-upgrades",
            "/var/log/apport.log",
            "/var/log/landscape"
        ])


# vim: et ts=4 sw=4



# Copyright (C) 2013 Red Hat, Inc., Lukas Zapletal <lzap@redhat.com>

# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.

# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.

# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

from sos.plugins import Plugin, RedHatPlugin


class Katello(Plugin, RedHatPlugin):
    """Katello project related information
    """

    plugin_name = 'katello'
    packages = ('katello', 'katello-common', 'katello-headpin')

    def setup(self):
        self.add_cmd_output("katello-debug --notar -d %s"
                            % self.get_cmd_output_path(name="katello-debug"))

# vim: et ts=4 sw=4



# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.

# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.

# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

from sos.plugins import Plugin, RedHatPlugin


class SELinux(Plugin, RedHatPlugin):
    """selinux related information
    """

    plugin_name = 'selinux'

    option_list = [("fixfiles", 'Print incorrect file context labels',
                    'slow', False),
                   ("list", 'List objects and their context', 'slow', False)]
    packages = ('libselinux',)

    def setup(self):
        # sestatus is always collected in check_enabled()
        self.add_copy_spec("/etc/selinux")
        self.add_cmd_outputs([
            "sestatus -b",
            "semodule -l",
            "selinuxdefcon root",
            "selinuxconlist root",
            "selinuxexeccon /bin/passwd",
            "semanage -o -"
        ])
        if self.get_option('fixfiles'):
            self.add_cmd_output("fixfiles -v check")
        if self.get_option('list'):
            self.add_cmd_outputs([
                "semanage fcontext -l",
                "semanage user -l",
                "semanage login -l",
                "semanage port -l"
            ])


# vim: et ts=4 sw=4



#!/usr/bin/env python

from distutils.core import setup
from distutils.command.build import build
from distutils.command.install_data import install_data
from distutils.dep_util import newer
from distutils.log import warn, info, error

import glob
import os
import subprocess
import sys

from sos import __version__ as VERSION

PO_DIR = 'po'
MO_DIR = os.path.join('build', 'mo')

class BuildData(build):
  def run(self):
    build.run(self)
    for po in glob.glob(os.path.join(PO_DIR, '*.po')):
      lang = os.path.basename(po[:-3])
      mo = os.path.join(MO_DIR, lang, 'sos.mo')

      directory = os.path.dirname(mo)
      if not os.path.exists(directory):
        os.makedirs(directory)

      if newer(po, mo):
        try:
          rc = subprocess.call(['msgfmt', '-o', mo, po])
          if rc != 0:
            raise Warning("msgfmt returned %d" % (rc,))
        except Exception as e:
          error("Failed gettext.")
          sys.exit(1)

class InstallData(install_data):
  def run(self):
    self.data_files.extend(self._find_mo_files())
    install_data.run(self)

  def _find_mo_files(self):
    data_files = []
    for mo in glob.glob(os.path.join(MO_DIR, '*', 'sos.mo')):
      lang = os.path.basename(os.path.dirname(mo))
      dest = os.path.join('share', 'locale', lang, 'LC_MESSAGES')
      data_files.append((dest, [mo]))
    return data_files

setup(name='sosreport',
      version=VERSION,
      description="""Set of tools to gather troubleshooting data
      from a system Sos is a set of tools that gathers information about system 
      hardware and configuration. The information can then be used for
      diagnostic purposes and debugging. Sos is commonly used to help
      support technicians and developers.""",
      author='Bryn M. Reeves',
      author_email='bmr@redhat.com',
      url='https://github.com/sosreport/sos',
      license="GPLv2+",
      scripts=['sosreport'],
      data_files=[
        ('share/man/man1', ['man/en/sosreport.1']),
        ('share/man/man5', ['man/en/sos.conf.5']),
        ],
      packages=['sos', 'sos.plugins', 'sos.policies'],
      cmdclass={'build': BuildData, 'install_data': InstallData},
      requires=['six'],
     )


# vim: et ts=4 sw=4



# Copyright 2010 Red Hat, Inc.
# Author: Adam Stokes <astokes@fedoraproject.org>

# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.

# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.

# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.


"""
This module houses the i18n setup and message function. The default is to use
gettext to internationalize messages.
"""

__version__ = "3.2.0a1"

import gettext
gettext_dir = "/usr/share/locale"
gettext_app = "sos"

gettext.bindtextdomain(gettext_app, gettext_dir)


def _default(msg):
    return gettext.dgettext(gettext_app, msg)

_sos = _default



# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.

# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.

# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

from sos.plugins import Plugin, RedHatPlugin, DebianPlugin, UbuntuPlugin


class KDump(Plugin):
    """Kdump related information
    """

    plugin_name = "kdump"

    def setup(self):
        self.add_copy_specs([
            "/proc/cmdline"
        ])


class RedHatKDump(KDump, RedHatPlugin):
    """Kdump related information for Red Hat distributions
    """

    files = ('/etc/kdump.conf',)
    packages = ('kexec-tools',)

    def setup(self):
        self.add_copy_specs([
            "/etc/kdump.conf",
            "/etc/udev/rules.d/*kexec.rules",
            "/var/crash/*/vmcore-dmesg.txt"
        ])


class DebianKDump(KDump, DebianPlugin, UbuntuPlugin):
    """Kdump related information for Debian distributions
    """

    files = ('/etc/default/kdump-tools',)
    packages = ('kdump-tools',)

    def setup(self):
        self.add_copy_specs([
            "/etc/default/kdump-tools"
        ])

# vim: et ts=4 sw=4



# Copyright (C) 2007 Red Hat, Inc., Eugene Teo <eteo@redhat.com>

# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.

# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.

# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

from sos.plugins import Plugin, RedHatPlugin, DebianPlugin, UbuntuPlugin


class Sendmail(Plugin):
    """sendmail information
    """

    plugin_name = "sendmail"

    packages = ('sendmail',)


class RedHatSendmail(Sendmail, RedHatPlugin):
    """sendmail information for RedHat based distributions
    """

    files = ('/etc/rc.d/init.d/sendmail',)
    packages = ('sendmail',)

    def setup(self):
        super(RedHatSendmail, self).setup()
        self.add_copy_specs([
            "/etc/mail/*",
            "/var/log/maillog"
        ])


class DebianSendmail(Sendmail, DebianPlugin, UbuntuPlugin):
    """sendmail information for Debian based distributions
    """

    files = ('/etc/init.d/sendmail',)
    packages = ('sendmail',)

    def setup(self):
        super(DebianSendmail, self).setup()
        self.add_copy_specs([
            "/etc/mail/*",
            "/var/log/mail.*"
        ])

# vim: et ts=4 sw=4



# Copyright (C) 2013 Red Hat, Inc., Flavio Percoco <fpercoco@redhat.com>
# Copyright (C) 2012 Rackspace US, Inc.,
#               Justin Shepherd <jshepher@rackspace.com>
# Copyright (C) 2009 Red Hat, Inc., Joey Boggs <jboggs@redhat.com>

# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.

# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.

# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

from sos.plugins import Plugin, RedHatPlugin, DebianPlugin, UbuntuPlugin


class OpenStackSwift(Plugin):
    """OpenstackSwift related information."""
    plugin_name = "openstack-swift"

    option_list = [("log", "gathers openstack-swift logs", "slow", False)]

    def setup(self):
        # Swift
        self.add_copy_spec("/etc/swift/")


class DebianOpenStackSwift(OpenStackSwift, DebianPlugin, UbuntuPlugin):
    """OpenStackSwift related information for Debian based distributions."""

    packages = (
        'swift',
        'swift-account',
        'swift-container',
        'swift-object',
        'swift-proxy',
        'swauth',
        'python-swift',
        'python-swauth'
    )


class RedHatOpenStackSwift(OpenStackSwift, RedHatPlugin):
    """OpenStackSwift related information for Red Hat distributions."""

    packages = (
        'openstack-swift',
        'openstack-swift-account',
        'openstack-swift-container',
        'openstack-swift-object',
        'openstack-swift-proxy',
        'swift',
        'python-swiftclient'
    )

# vim: et ts=4 sw=4



from sos.policies import Policy
from sos.utilities import shell_out


class OSXPolicy(Policy):

    distro = "Mac OS X"

    @classmethod
    def check(class_):
        try:
            return "Mac OS X" in shell_out("sw_vers")
        except Exception:
            return False

# vim: et ts=4 sw=4



# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.

# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.

# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

from sos.plugins import Plugin, RedHatPlugin, DebianPlugin, UbuntuPlugin


class Apache(Plugin):
    """Apache related information
    """
    plugin_name = "apache"

    option_list = [
        ("log", "gathers all apache logs", "slow", False)
    ]


class RedHatApache(Apache, RedHatPlugin):
    """Apache related information for Red Hat distributions
    """
    files = ('/etc/httpd/conf/httpd.conf',)

    def setup(self):
        super(RedHatApache, self).setup()

        self.add_copy_specs([
            "/etc/httpd/conf/httpd.conf",
            "/etc/httpd/conf.d/*.conf"
        ])

        self.add_forbidden_path("/etc/httpd/conf/password.conf")

        # collect only the current log set by default
        self.add_copy_spec_limit("/var/log/httpd/access_log", 5)
        self.add_copy_spec_limit("/var/log/httpd/error_log", 5)
        self.add_copy_spec_limit("/var/log/httpd/ssl_access_log", 5)
        self.add_copy_spec_limit("/var/log/httpd/ssl_error_log", 5)
        if self.get_option("log"):
            self.add_copy_spec("/var/log/httpd/*")


class DebianApache(Apache, DebianPlugin, UbuntuPlugin):
    """Apache related information for Debian distributions
    """
    files = ('/etc/apache2/apache2.conf',)

    def setup(self):
        super(DebianApache, self).setup()
        self.add_copy_specs([
            "/etc/apache2/*",
            "/etc/default/apache2"
        ])

        # collect only the current log set by default
        self.add_copy_spec_limit("/var/log/apache2/access_log", 15)
        self.add_copy_spec_limit("/var/log/apache2/error_log", 15)
        if self.get_option("log"):
            self.add_copy_spec("/var/log/apache2/*")

# vim: et ts=4 sw=4



# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.

# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.

# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

from sos.plugins import Plugin, RedHatPlugin
import os


class Anaconda(Plugin, RedHatPlugin):
    """Anaconda / Installation information
    """

    plugin_name = 'anaconda'

    files = (
        '/var/log/anaconda.log',
        '/var/log/anaconda'
    )

    def setup(self):

        paths = [
            "/root/anaconda-ks.cfg"
        ]

        if os.path.isdir('/var/log/anaconda'):
            # new anaconda
            paths.append('/var/log/anaconda')
        else:
            paths = paths + [
                "/var/log/anaconda.*"
                "/root/install.log",
                "/root/install.log.syslog"
            ]

        self.add_copy_specs(paths)

    def postproc(self):
        self.do_file_sub(
            "/root/anaconda-ks.cfg",
            r"(\s*rootpw\s*).*",
            r"\1********"
        )
        self.do_file_sub(
            "/root/anaconda-ks.cfg",
            r"(useradd --password) (.*)",
            r"\1 ********"
        )

# vim: et ts=4 sw=4



# Copyright (C) 2012 Red Hat, Inc.,
#   Jesse Jaggars <jjaggars@redhat.com>
#   Bryn M. Reeves <bmr@redhat.com>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.

# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.

# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
import os
import time
import tarfile
import zipfile
import shutil
import logging
import shlex
import re
import codecs
# required for compression callout (FIXME: move to policy?)
from subprocess import Popen, PIPE

try:
    import selinux
except ImportError:
    pass

# PYCOMPAT
import six
if six.PY3:
    long = int


class Archive(object):
    """Abstract base class for archives."""

    @classmethod
    def archive_type(class_):
        """Returns the archive class's name as a string.
        """
        return class_.__name__

    log = logging.getLogger("sos")

    _name = "unset"
    _debug = False

    def _format_msg(self, msg):
        return "[archive:%s] %s" % (self.archive_type(), msg)

    def set_debug(self, debug):
        self._debug = debug

    def log_error(self, msg):
        self.log.error(self._format_msg(msg))

    def log_warn(self, msg):
        self.log.warning(self._format_msg(msg))

    def log_info(self, msg):
        self.log.info(self._format_msg(msg))

    def log_debug(self, msg):
        if not self._debug:
            return
        self.log.debug(self._format_msg(msg))

    # this is our contract to clients of the Archive class hierarchy.
    # All sub-classes need to implement these methods (or inherit concrete
    # implementations from a parent class.
    def add_file(self, src, dest=None):
        raise NotImplementedError

    def add_string(self, content, dest):
        raise NotImplementedError

    def add_link(self, source, link_name):
        raise NotImplementedError

    def add_dir(self, path):
        raise NotImplementedError

    def add_node(self, path, mode, device):
        raise NotImplementedError

    def get_tmp_dir(self):
        """Return a temporary directory that clients of the archive may
        use to write content to. The content of the path is guaranteed
        to be included in the generated archive."""
        raise NotImplementedError

    def get_archive_path(self):
        """Return a string representing the path to the temporary
        archive. For archive classes that implement in-line handling
        this will be the archive file itself. Archives that use a
        directory based cache prior to packaging should return the
        path to the temporary directory where the report content is
        located"""
        pass

    def cleanup(self):
        """Clean up any temporary resources used by an Archive class."""
        pass

    def finalize(self, method):
        """Finalize an archive object via method. This may involve creating
        An archive that is subsequently compressed or simply closing an
        archive that supports in-line handling. If method is automatic then
        the following methods are tried in order: xz, bz2 and gzip"""

        self.close()


class FileCacheArchive(Archive):
    """ Abstract superclass for archive types that use a temporary cache
    directory in the file system. """

    _tmp_dir = ""
    _archive_root = ""
    _archive_name = ""

    def __init__(self, name, tmpdir):
        self._name = name
        self._tmp_dir = tmpdir
        self._archive_root = os.path.join(tmpdir, name)
        os.makedirs(self._archive_root, 0o700)
        self.log_info("initialised empty FileCacheArchive at '%s'" %
                      (self._archive_root,))

    def dest_path(self, name):
        if os.path.isabs(name):
            name = name.lstrip(os.sep)
        return (os.path.join(self._archive_root, name))

    def _check_path(self, dest):
        dest_dir = os.path.split(dest)[0]
        if not dest_dir:
            return
        if not os.path.isdir(dest_dir):
            self._makedirs(dest_dir)

    def add_file(self, src, dest=None):
        if not dest:
            dest = src
        dest = self.dest_path(dest)
        self._check_path(dest)
        try:
            shutil.copy(src, dest)
        except IOError as e:
            self.log_info("caught '%s' copying '%s'" % (e, src))
        try:
            shutil.copystat(src, dest)
        except OSError:
            # SELinux xattrs in /proc and /sys throw this
            pass
        try:
            stat = os.stat(src)
            os.chown(dest, stat.st_uid, stat.st_gid)
        except Exception as e:
            self.log_debug("caught '%s' setting ownership of '%s'" % (e, dest))
        self.log_debug("added '%s' to FileCacheArchive '%s'" %
                       (src, self._archive_root))

    def add_string(self, content, dest):
        src = dest
        dest = self.dest_path(dest)
        self._check_path(dest)
        f = codecs.open(dest, 'w', encoding='utf-8')
        f.write(content)
        if os.path.exists(src):
            try:
                shutil.copystat(src, dest)
            except OSError as e:
                self.log_error(
                    "Unable to add '%s' to FileCacheArchive: %s" % (dest, e))
        self.log_debug("added string at '%s' to FileCacheArchive '%s'"
                       % (src, self._archive_root))

    def add_link(self, source, link_name):
        dest = self.dest_path(link_name)
        self._check_path(dest)
        if not os.path.exists(dest):
            os.symlink(source, dest)
        self.log_debug("added symlink at '%s' to '%s' in FileCacheArchive '%s'"
                       % (dest, source, self._archive_root))

    def add_dir(self, path):
        self.makedirs(path)

    def add_node(self, path, mode, device):
        dest = self.dest_path(path)
        self._check_path(dest)
        if not os.path.exists(dest):
            os.mknod(dest, mode, device)
            shutil.copystat(path, dest)

    def _makedirs(self, path, mode=0o700):
        os.makedirs(path, mode)

    def get_tmp_dir(self):
        return self._archive_root

    def get_archive_path(self):
        return self._archive_root

    def makedirs(self, path, mode=0o700):
        self._makedirs(self.dest_path(path))
        self.log_debug("created directory at '%s' in FileCacheArchive '%s'"
                       % (path, self._archive_root))

    def open_file(self, path):
        path = self.dest_path(path)
        return codecs.open(path, "r", encoding='utf-8')

    def cleanup(self):
        shutil.rmtree(self._archive_root)

    def finalize(self, method):
        self.log_info("finalizing archive '%s' using method '%s'"
                      % (self._archive_root, method))
        self._build_archive()
        self.cleanup()
        self.log_info("built archive at '%s' (size=%d)" % (self._archive_name,
                      os.stat(self._archive_name).st_size))
        self.method = method
        return self._compress()


class TarFileArchive(FileCacheArchive):
    """ archive class using python TarFile to create tar archives"""

    method = None
    _with_selinux_context = False

    def __init__(self, name, tmpdir):
        super(TarFileArchive, self).__init__(name, tmpdir)
        self._suffix = "tar"
        self._archive_name = os.path.join(tmpdir, self.name())

    def set_tarinfo_from_stat(self, tar_info, fstat, mode=None):
        tar_info.mtime = fstat.st_mtime
        tar_info.pax_headers['atime'] = "%.9f" % fstat.st_atime
        tar_info.pax_headers['ctime'] = "%.9f" % fstat.st_ctime
        if mode:
            tar_info.mode = mode
        else:
            tar_info.mode = fstat.st_mode
        tar_info.uid = fstat.st_uid
        tar_info.gid = fstat.st_gid

    # this can be used to set permissions if using the
    # tarfile.add() interface to add directory trees.
    def copy_permissions_filter(self, tarinfo):
        orig_path = tarinfo.name[len(os.path.split(self._name)[-1]):]
        if not orig_path:
            orig_path = self._archive_root
        try:
            fstat = os.stat(orig_path)
        except OSError:
            return tarinfo
        if self._with_selinux_context:
            context = self.get_selinux_context(orig_path)
            if(context):
                tarinfo.pax_headers['RHT.security.selinux'] = context
        self.set_tarinfo_from_stat(tarinfo, fstat)
        return tarinfo

    def get_selinux_context(self, path):
        try:
            (rc, c) = selinux.getfilecon(path)
            return c
        except:
            return None

    def name(self):
        return "%s.%s" % (self._name, self._suffix)

    def _build_archive(self):
        tar = tarfile.open(self._archive_name, mode="w")
        # we need to pass the absolute path to the archive root but we
        # want the names used in the archive to be relative.
        tar.add(self._archive_root, arcname=os.path.split(self._name)[1],
                filter=self.copy_permissions_filter)
        tar.close()

    def _compress(self):
        methods = ['xz', 'bzip2', 'gzip']
        if self.method in methods:
            methods = [self.method]

        last_error = Exception("compression failed for an unknown reason")

        for cmd in methods:
            suffix = "." + cmd.replace('ip', '')
            # use fast compression if using xz or bz2
            if cmd != "gzip":
                cmd = "%s -1" % cmd
            try:
                command = shlex.split("%s %s" % (cmd, self.name()))
                p = Popen(command,
                          stdout=PIPE,
                          stderr=PIPE,
                          bufsize=-1,
                          close_fds=True)
                stdout, stderr = p.communicate()
                if stdout:
                    self.log_info(stdout.decode('utf-8'))
                if stderr:
                    self.log_error(stderr.decode('utf-8'))
                self._suffix += suffix
                return self.name()
            except Exception as e:
                last_error = e
        raise last_error


class ZipFileArchive(Archive):
    """ archive class using python ZipFile to create zip archives """

    def __init__(self, name):
        self._name = name
        try:
            import zlib
            assert zlib
            self.compression = zipfile.ZIP_DEFLATED
        except:
            self.compression = zipfile.ZIP_STORED

        self.zipfile = zipfile.ZipFile(self.name(), mode="w",
                                       compression=self.compression)

    def name(self):
        return "%s.zip" % self._name

    def finalize(self, method):
        super(ZipFileArchive, self).finalize(method)
        return self.name()

    def add_file(self, src, dest=None):
        src = str(src)
        if dest:
            dest = str(dest)

        if os.path.isdir(src):
            # We may not need, this, but if we do I only want to do it
            # one time
            regex = re.compile(r"^" + src)
            for path, dirnames, filenames in os.walk(src):
                for filename in filenames:
                    filename = "/".join((path, filename))
                    if dest:
                        self.zipfile.write(filename, re.sub(regex, dest,
                                                            filename))
                    else:
                        self.zipfile.write(filename)
        else:
            if dest:
                self.zipfile.write(src, dest)
            else:
                self.zipfile.write(src)

    def add_string(self, content, dest):
        info = zipfile.ZipInfo(dest,
                               date_time=time.localtime(time.time()))
        info.compress_type = self.compression
        info.external_attr = 0o400 << long(16)
        self.zipfile.writestr(info, content)

    def open_file(self, name):
        try:
            self.zipfile.close()
            self.zipfile = zipfile.ZipFile(self.name(), mode="r")
            file_obj = self.zipfile.open(name)
            return file_obj
        finally:
            self.zipfile.close()
            self.zipfile = zipfile.ZipFile(self.name(), mode="a")

    def close(self):
        self.zipfile.close()

# vim: et ts=4 sw=4
